local Grid2Alert = Grid2:NewModule("Grid2Alert", "LibSink-2.0")

local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

local media = LibStub("LibSharedMedia-3.0")
media:Register("sound", "threat", "Interface\\Addons\\Grid2Alert\\sounds\\aggro.wav")
media:Register("sound", "detox", "Sound\\interface\\AuctionWindowOpen.wav")

local function Alert_Enable(self)
	local status = self.status
	if not self.HookName then
		if status.HasStateChanged then
			-- Aura Status don't call UpdateIndicators
			self.HookName = "HasStateChanged"
			self.hook = function (status, unit)
				local changed = self.prev(status, unit)
				if changed then
					self:Update(unit, status:IsActive(unit))
				end
			end
-- print("Alert_Enable", self.name, self.HookName, self.hook, status.name)
		else
			self.HookName = "UpdateIndicators"
			self.hook = function (status, unit)
				self.prev(status, unit)
				self:Update(unit, status:IsActive(unit))
			end
-- print("Alert_Enable", self.name, self.HookName, self.hook, status.name)
		end
		self.prev = status[self.HookName]
	end
	status[self.HookName] = self.hook
end

local function Alert_Disable(self)
	if not self.HookName then return end
	self.status[self.HookName] = self.prev
end

local function Alert_UpdateSelfGain(self, unit, isActive)
	if UnitIsUnit(unit, "player") then
		if not self.active and isActive then
			self:TriggerAlert(unit)
		end
		self.active = isActive
	end
end

local function Alert_UpdateSelfGainAggro(self, unit, isActive)
	if UnitIsUnit(unit, "player") then
		-- Override so that it only triggers when the player has aggro
		isActive = isActive and UnitThreatSituation(unit) > 1
		if not self.active and isActive then
			self:TriggerAlert(unit)
		end
		self.active = isActive
	end
end

local function Alert_UpdateSelfLost(self, unit, isActive)
	if UnitIsUnit(unit, "player") then
		if self.active and not isActive then
			self:TriggerAlert(unit)
		end
		self.active = isActive
	end
end

local function Alert_UpdateAnyGain(self, unit, isActive)
-- print("Alert_UpdateAnyGain", unit, isActive, self.count)
	--ToDo: Fix this to use a unit cache.
	-- if isActive and self.count == 0 then
		-- self:TriggerAlert(unit)
	-- end
	-- self.count = self.count + (isActive and 1 or -1)
	if isActive and self.count == 0 then
		self:TriggerAlert(unit)
	elseif (isActive) then
		self.count = self.count + 1
	else
		self.count = 0
	end
end

local function Alert_UpdateAllLost(self, unit, isActive)
	--ToDo: Fix this to use a unit cache.
	self.count = self.count + (isActive and 1 or -1)
	if not isActive and self.count == 0 then
		self:TriggerAlert(unit)
	end
end

local function Alert_TriggerAlert(self, unit)
	return Grid2Alert:TriggerAlert(self, unit)
end

local function Alert_Initialize(self, type)
	if type == "self-gain" then
		self.Update = Alert_UpdateSelfGain
	elseif type == "self-lost" then
		self.Update = Alert_UpdateSelfLost
	elseif type == "any-gain" then
		self.count = 0
		self.Update = Alert_UpdateAnyGain
	elseif type == "all-lost" then
		self.count = 0
		self.Update = Alert_UpdateAllLost
	elseif type == "self-gain-aggro" then
		self.Update = Alert_UpdateSelfGainAggro
	else
		Grid2:Print("ERROR ! invalid alert type", type)
	end
end

function Grid2Alert:HookStatus(status, name, dbx)
	if type(status) == "string" then
		status = Grid2.statuses[status]
-- print("Grid2Alert:HookStatus", status, name, dbx.type)
	end
	local Alert = {
		status = status,
		name = name,
		Enable = Alert_Enable,
		Disable = Alert_Disable,
		TriggerAlert = Alert_TriggerAlert,
		dbx = dbx,
	}

	Alert_Initialize(Alert, dbx.type)
	self.alerts[Alert] = status

	return Alert
end

function Grid2Alert:OnInitialize()
	self.core.defaultModulePrototype.OnInitialize(self)
	self.alerts = {}
	self.notification_times = {}

	-- media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateMedia")
end

function Grid2Alert:GetStatusList(status)
	if type(status) == "string" and type(self[status]) == "function" then
		return self[status](self)
	elseif type(status) == "table" then
		return status
	-- else
		-- return { status }
	end
end

local enabled
function Grid2Alert:OnEnable()
	for alert in pairs(self.alerts) do
		alert:Enable()
	end
end

function Grid2Alert:OnDisable()
	for alert in pairs(self.alerts) do
		alert:Disable()
	end
end

function Grid2Alert:TriggerAlert(alert, unit)
	local p = self.dbx
	local name = alert.name
	local dbx = alert.dbx
	if (not dbx) then
		self:Debug("Invalid alert played : \"%s\", I don't know it\n", name)
		return
	end
	local cur_time = GetTime()

	if (cur_time - (self.last_play_time or 0) < p.minPause or
	   cur_time - (self.notification_times[name] or 0) < dbx.minPause) then
		self:Debug("Skipping alert \"%s\"", name)
	else
		self.last_play_time = cur_time
		self.notification_times[name] = cur_time

		self:SendMessage("GridAlert_Notify", name)
		if dbx.sound then
			PlaySoundFile(media:Fetch("sound", dbx.sound))
		end
        local message = dbx.message
		if message then
			if message:find("%t", nil, true) then
				message = message:gsub("%%t", UnitName(unit))
			end
			self:Pour(message, 1, 0, 0)
		end
	end
end


-- Hook the loading of setup so our objects can get initialized.
local prev_SetupCustom = Grid2.SetupCustom
function Grid2:SetupCustom(setup, objects, profileCurrentKey, ...)
	prev_SetupCustom(self, setup, objects, profileCurrentKey, ...)

	Grid2Alert.dbx = DBL:GetRuntimeGlobals(Grid2.dblData, "alerts")
	Grid2Alert:SetSinkStorage(Grid2Alert.dbx.sinkOptions)
	Grid2.Grid2Alert = Grid2Alert
	Grid2.alerts = {}
	
	local alertObjects = objects.alerts
	for baseKey, layer in pairs(setup.alerts[profileCurrentKey]) do
		local dbx = alertObjects[layer][baseKey]
		Grid2.alerts[baseKey] = {dbx = dbx, alerts = {}}
		local alerts = Grid2.alerts[baseKey].alerts
-- print("SetupCustom Alerts:", layer, baseKey, dbx.type)
		local statusList = Grid2Alert:GetStatusList(dbx.status)
		if statusList then
			for index, statusKey in ipairs(statusList) do
				local alert = Grid2Alert:HookStatus(statusKey, baseKey, dbx)
				alerts[statusKey] = alert
			end
			
		end
	end

	for alert in pairs(Grid2Alert.alerts) do
		alert:Enable()
	end
end

-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	upgrade = DBL:LoadOptions("Grid2AlertOptions", dblData, nil, "account", 1, Grid2.dblData.classKey, 1, "globals", 1) or upgrade

	return upgrade
end

-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)

	local Grid2AlertOptions = Grid2Options.plugins["Grid2AlertOptions"]
	if (Grid2AlertOptions) then
		flatten = DBL:UpgradeDefaults("Grid2AlertOptions", dblData, Grid2AlertOptions.UpgradeDefaults, "account", 1, Grid2.dblData.classKey, 1, "globals", 1) or flatten
	end
	
	return flatten
end
